﻿using anbs_cp.Interfaces;

namespace anbs_cp.Classes;

/// <summary>
/// Сериализованная модель значения класса, хранящего в себе состояния действия
/// </summary>
public class ActionStateSerializableValue: ISerializable
{
    /// <summary>
    /// Значение - null?
    /// </summary>
    private bool IsNull { get; set; }

    /// <summary>
    /// Сериализуемое значение
    /// </summary>
    private string Serialized { get; set; } = string.Empty;

    /// <summary>
    /// Получает значение из <paramref name="value"/>
    /// </summary>
    /// <param name="value">Значение</param>
    /// <typeparam name="T">Тип значения</typeparam>
    public void GetValue<T> (T? value)
    {
        // Значение - null?
        IsNull = value == null;

        // Если не null
        if (value != null)
        {
            // - значение поддерживает интерфейс IsSerialized
            if (value is ISerializable serializableValue)
                // -- сериализую
                Serialized = serializableValue.Serialize();
            else
                // -- сериализую
                Serialized = new SysTextSerializer().Serialize(value);
        }
        else
            // Сериализую
            Serialized = new SysTextSerializer().Serialize(value);
    }

    /// <summary>
    /// Устанавливает значение
    /// </summary>
    /// <typeparam name="T">Класс</typeparam>
    /// <returns>Значение</returns>
    public T? SetValue<T> ()
    {
        // Если null
        if (IsNull)
            // - то возыращаем null
            return default;

        // Если тип T не реализует интерфейс ISerializable
        if (typeof(T).GetInterface(nameof(ISerializable)) == null)
            // - то воспользуемся простой десериализацией
            return new SysTextSerializer().Deserialize<T>(Serialized);

        // Создаю модель
        T? model = default;

        // Десериализую модель
        (model as ISerializable)?.Deserialize(Serialized);

        // Возвращаю модель
        return model!;

        // Если null
    }

    #region Реализация интерфейса ISerializable
    /// <inheritdoc />
    public string Serialize () => new SysTextSerializer().Serialize(this);

    /// <inheritdoc />
    public void Deserialize (string json)
    {
        // Десериализую строку
        ActionStateSerializableValue item = new SysTextSerializer().Deserialize<ActionStateSerializableValue>(json) ?? new();

        // Передаю параметры
        IsNull = item.IsNull;
        Serialized = item.Serialized;
    }
    #endregion
}